//editorwin.cpp
//implementation file of the EditorWindow class

#include "editorwin.h"
#include "cpath.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qsimplerichtext.h>
#include <qfont.h>

#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"
#include "console.xpm"

EditorWindow::EditorWindow() : QMainWindow(0, "FEdit 1.0", WGroupLeader)
{
  //--------------Variable declarations-------------------
  
  int windowHeight, windowWidth;
  fontSize = 10; //used later in QTextEdit widget set up.
  
  //Printer declaration
  printer = new QPrinter(QPrinter::HighResolution);
  
  //Button menu items & pixmaps

  //The Button Icons
  QPixmap openIcon, saveIcon, printIcon, consoleIcon;
  openIcon = QPixmap(fileopen);
  saveIcon = QPixmap(filesave);
  printIcon = QPixmap(fileprint);
  consoleIcon = QPixmap(console_xpm);

  //adding icons to the toolbar.
  QToolBar *fileTools = new QToolBar(this, "file operations");  
  fileTools->setLabel("File Operations");
  
  QToolButton *fileOpen = new QToolButton(openIcon, "Open File", QString::null,
  			this, SLOT(choose()), fileTools, "open file");
  QToolButton *fileSave = new QToolButton(saveIcon, "Save File", QString::null,
			  this, SLOT(save()), fileTools, "save file");
  QToolButton *filePrint = new QToolButton(printIcon, "Print File", QString::null,
  			   this, SLOT(print()), fileTools, "print file");
  QToolButton *console = new QToolButton(consoleIcon, "Open Console", QString::null,
  			   this, SLOT(console()), fileTools, "open console");
  
    //placement of buttons on bar.  
  QMimeSourceFactory::defaultFactory()->setPixmap("fileopen", openIcon);

 
  //Drop down menus
  QPopupMenu *file = new QPopupMenu(this);
  QPopupMenu *optionsMenu = new QPopupMenu(this);
  QPopupMenu *consoleMenu = new QPopupMenu(this);
  QPopupMenu *help = new QPopupMenu(this);

  //---------end of variable declaration--------
  
  //---------load saved settings----------------
  
  QString settingsFileName = "fedit.conf";
  QFile settingsFile(settingsFileName);
  
  if (settingsFile.open(IO_ReadOnly))
  {
    QTextStream settingsInfo(&settingsFile);
    //windowWidth  = settingsInfo.read();
    //windowHeight = settingsInfo.read();
    statusBar()->message("Reading user settings...", 2000);
    settingsFile.close();
  }
  
  else
  {
    statusBar()->message("Error loading user settings! Using "
    	"defaults", 2000);
    //default settings:
    windowWidth  = 500;
    windowHeight = 500;
  }

  //---------end of loading user settings-------------
  
  //---------Pull down menus--------------------------

  //File Menu
    
  menuBar()->insertItem("&File", file);
  
  file->insertItem("&New", this, SLOT(newFile()), CTRL+Key_N);
  file->insertItem(openIcon, "&Open...", this, SLOT(choose()), CTRL+Key_O);
  file->insertItem(saveIcon, "&Save", this, SLOT(save()), CTRL+Key_S);
  file->insertSeparator();
  file->insertItem(printIcon, "&Print...", this, SLOT(print()), CTRL+Key_P);
  file->insertSeparator();
  file->insertItem("&Close", this, SLOT(close()), CTRL+Key_W);
  file->insertItem("&Quit", qApp, SLOT(closeAllWindows()), CTRL+Key_Q);
  
  menuBar()->insertSeparator();
  
  //Options Menu
  
  menuBar()->insertItem("&Options", optionsMenu);
  
  optionsMenu->insertItem("&Increase Font", this, SLOT(zoomFontIn()), CTRL+Key_I);
  optionsMenu->insertItem("&Decrease Font", this, SLOT(zoomFontOut()), CTRL+Key_D);
  
  //Console menu
  
  menuBar()->insertItem("&Console", consoleMenu);
  
  consoleMenu->insertItem("&Open Console", this, SLOT(console()), CTRL+Key_C);
  consoleMenu->insertItem("Console &Path", this, SLOT(setConsole()));
  
  menuBar()->insertSeparator();
  
  //Help Menu
  
  
  menuBar()->insertItem("&Help", help);
  
  help->insertItem("&About", this, SLOT(about()), Key_F1);
  help->insertItem("About &Qt", this, SLOT(aboutQt()));

  //----------end of pull down menus----------------
  
  //----------widget creation----------------------

  //the creation of the actual widget.
  e = new QTextEdit(this, "editor");
  e->setFocus();
  setCentralWidget(e);
  statusBar()->message("Ready for input.", 2000);
  
  //setting up widget and set size of window using loaded user settings 
  e->setCurrentFont((const QFont)"Courier");
  e->setPointSize(fontSize);
  resize(windowHeight, windowWidth);
  
  //-----------end of widget creation---------------
  
}

EditorWindow::~EditorWindow()
{
  delete printer;
}

void EditorWindow::newFile()
{

  EditorWindow *ed = new EditorWindow;
  ed->setCaption("NE File Editor v1.0");
  ed->show();
  /*
  Needs work, trying to make it so that new files do not open in a new window.. that is
  annoying.
  */

  if (e->isModified())
  {
    statusBar()->message("MODIFED FILE!", 5000);
      
    switch(QMessageBox::information(this, "NE File Editor v1.0",
    				  "Do you want to save the changes to the document?",
				  "Yes", "No", "Cancel", 0, 1))
    {	
      case 0:
        save();
        break;
       
      case 1:
        filename = "";
	e->text() = "";
        break;
       
      case 2:
      default: 
	return;
        break;
    }

  }
  else
  {
    statusBar()->message("UNMODIFIED!", 5000);
  }
}

void EditorWindow::choose()
{
  QString fn = QFileDialog::getOpenFileName(QString::null, QString::null, this);
  
  if (!fn.isEmpty())
    load(fn);
  else
    statusBar()->message("Loading aborted", 2000);
}

void EditorWindow::load(const QString &fileName)
{
  QFile f(fileName);
  if (!f.open(IO_ReadOnly))
    return;
    
  QTextStream ts(&f);
  e->setText(ts.read());
  e->setModified(FALSE);
  setCaption(fileName);
  statusBar()->message("Loaded document " + fileName, 2000);
}

void EditorWindow::save()
{
  if (filename.isEmpty())
  {
    saveAs();
    return;
  }
  
  QString text = e->text();
  QFile f(filename);
  if (!f.open(IO_WriteOnly))
  {
    statusBar()->message(QString("Could not write to %l").arg(filename), 2000);
    return;
  }
  QTextStream t(&f);
  t << text;
  f.close();
  
  e->setModified(FALSE);
  setCaption(filename);
  statusBar()->message(QString("File %l saved").arg(filename), 2000);
}

void EditorWindow::saveAs()
{
  QString fn = QFileDialog::getSaveFileName(QString::null, QString::null, this);
  
  if (!fn.isEmpty())
  {
    filename = fn;
    save();
  }
  else
  {
    statusBar()->message("Saving aborted", 2000);
  }
}

void EditorWindow::print()
{
  printer->setFullPage(TRUE);
  if (printer->setup(this))
  {
    statusBar()->message("Printing...");
    QPainter p;
    if (!p.begin(printer))
    {
      statusBar()->message("Printing aborted", 2000);
      return;
    }
    
    QPaintDeviceMetrics metrics(p.device());
    int dpiy = metrics.logicalDpiY();
    int margin = (int) ((2/2.54)*dpiy); //2 cm margins
    
    QRect view(margin, margin, metrics.width() - 2 * margin, metrics.height() - 2 * margin);
    
    QSimpleRichText richText(QStyleSheet::convertFromPlainText(e->text()),
    			     QFont(),
			     e->context(),
			     e->styleSheet(),
			     e->mimeSourceFactory(),
			     view.height());

    richText.setWidth(&p, view.width());
    int page = 1;
    do
    {
      richText.draw(&p, margin, margin, view, colorGroup());
      view.moveBy(0, view.height());
      p.translate(0, -view.height());
      p.drawText(view.right() - p.fontMetrics().width(QString::number(page)), 
     	         view.bottom() + p.fontMetrics().ascent() + 5,
	         QString::number(page));
	       
      if (view.top() - margin >= richText.height())
        break;
      printer->newPage();
      page++;
    }while (TRUE);
  
    statusBar()->message("Printing completed", 2000);
  
  }
  
  else
  {
    statusBar()->message("Printing aborted", 2000);
  }
}

void EditorWindow::closeEvent(QCloseEvent *ce)
{
  if (!e->isModified())
  {
    ce->accept();
    return;
  }
  
  switch(QMessageBox::information(this, "NE File Editor v1.0",
  				  "Do you want to save the changes to the document?",
				  "Yes", "No", "Cancel", 0, 1))
  {
  
     case 0:
       save();
       ce->accept();
       break;
       
     case 1:
       ce->accept();
       break;
       
     case 2:
     default: 
       ce->ignore();
       break;
   }
}  
  
void EditorWindow::about()
{
  QMessageBox::about(this, "About NE File Editor v1.0",
  		     "This is a simple plain text file editor that allow you to edit multiple text files\n "
		     "at a once, there is also a built in command prompt loader for easy compilation of\n "
		     "any source files you may be working on.\n\n "
		     "This version isn't all that user friendly or that great. I didn't get to do all that\n"
		     "I wanted to do b/c I just plain don't know how! :) I hope someone finds some use for\n"
		     "this program and enjoys it. Oh and the console does work. It opens an external\n"
		     "terminal program of your choice to use. (Path is set in Console menu) If it's not\n"
		     "working, there's most likely a typograpical error in your path. Check it and try\n"
                     "again.\n\n"
		     "Anyway, have fun! Feel free to play around with the source, just give me some sort\n"
		     "credit. Also please send any comments of found bugs/errrors to the E-mail below\n"
		     "or to the website. Thanks!\n\n"
		     "By: Erik Eriksen\n"
		     "E-mail: eriker@farmingdale.edu\n"
		     "Web Site: http://eriks.servehttp.com\n"
		     "                http://www.geocities.com/nukemn0w/  (old site)\n");

}

void EditorWindow::aboutQt()
{
  QMessageBox::aboutQt(this, "NE File Editor v1.0");
}

void EditorWindow::console()
{

  QFile pathInfo("cpath.info");
  
  if (!pathInfo.open(IO_ReadOnly))
  {
    QMessageBox::critical(this, "ERROR!", "Error opening console path"
    " file!\nPlease check to see if cpath.info is present in current working\n"
    "directory and that a valid path is set. If problem presists, please\n"
    "see the \"about\" section on how to send bug information.");
    return;
  }
  QTextStream gettingInfo(&pathInfo);
  cPath = gettingInfo.read();

  
  system(cPath);         //pretty sneaky eh?
}

void EditorWindow::setConsole()
{
  ConsolePath *cp = new ConsolePath;
  
  cp->setCaption("Set Console Working Path");
  cp->show();
  
}

		     


void EditorWindow::zoomFontIn()
{
  fontSize++;
  if (fontSize > 72)
    fontSize = 72;
      
  e->setPointSize(fontSize);
  return;
}

void EditorWindow::zoomFontOut()
{
  fontSize--;
  if (fontSize < 4)
    fontSize = 4;
    
  e->setPointSize(fontSize);
  return;
}
  
